# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 3: Simulation: Random Values
#
# Time-stamp: <2008-03-13 22:34:13 chl>

n <- 1000
k <- 29983
a0 <- 2^10
m <- 2^20
ran0 <- NULL
for (i in 1:n) {
  k <- (a0+3)*k
  k <- k%%m
  ran0[i] <- k/(m-1)
}

set.seed(111)
round(runif(9),3)
set.seed(111)
round(runif(9),3)

round(runif(9),3)

###
### not in the book
ran1 <- NULL
for (i in 1:n) {
  k <- (a0+1)*k
  k <- k%%m
  ran1[i] <- k/(m-1)
}
ran2 <- runif(1000)

source("../chap_2/2.R") # for `describe` function
###

describe(ran0)
describe(ran1)
describe(ran2)

tab <- table(cut(ran0,seq(0,1,0.1)))
tab

x2 <- sum((tab-n/10)^2/(n/10))
probability <- 1-pchisq(x2,9)
cbind(x2,probability)

n <- 1000
x <- runif(n)
test <- diff(sign(diff(x)))
r <- sum(ifelse(test!=0,1,0)+1)
r0 <- (2*n-3)/3
v <- (16*n-29)/90
z <- (r-r0)/sqrt(v)
cat("r =",r,"r0 =",round(r0,3),"z =",round(z,3),"\n")
cat("p-value =",round(pnomr(z),3),"\n")

### - R --
### this cannot work since y has to be of length length(x)+1
stepfun(x,y)

                                        # Fig. 3.1
x <- c(0.15,0.16,0.18,0.22,0.23,0.29,0.34,0.65,0.77,0.92)
n <- length(x)
x <- sort(x)
y <- x
plot(x,y,pch="+",xlim=c(0,1),ylim=c(0,1),main="Kolmogorov-test -- plot")
abline(0,1,lty=3)
par(new=T)
y0 <- (1:n)/n
### - R --
### suppress type="l" and add main="" when calling next plot
plot(stepfun(c(0.15,x),c(0,0.1,y0)),xlim=c(0,1),ylim=c(0,1),
     xlab="",ylab="",main="",axes=F)
abline(h=0)
abline(h=1)
dif <- abs(y-y0)
m <- order(dif)[n]
### - R --
### add code=3 to arrows() instead of calling this function two times
arrows(x[m],y[m],x[m],y0[m],code=3,length=0.1)

K <- max(abs(y-(1:n)/n))
cat("maximum distance = ",round(K,3),"\n")

n <- 2000
y <- sort(runif(n))
K <- max(abs(y-(1:n)/n))
K20 <- 1.07/sqrt(n)
K10 <- 1.22/sqrt(n)
K5 <- 1.36/sqrt(n)
K1 <- 1.63/sqrt(n)
round(cbind(K,K20,K10,K5),3)

y <- sort(ran0)
n <- length(y)
K <- max(abs(y-(1:n)/n))
K20 <- 1.07/sqrt(n)
K10 <- 1.22/sqrt(n)
K5 <- 1.36/sqrt(n)
K1 <- 1.63/sqrt(n)
round(cbind(K,K20,K10,K5),3)

                                        # Fig. 3.2
###
### not in the book
plot(ran0[seq(1,length(ran0),by=2)],ran0[seq(2,length(ran0),by=2)],pch=".",
     xlab="even random values",ylab="odd random values",
     main="Random values from ran0")
###

                                        # Fig. 3.3
###
### not in the book
plot(ran1[seq(1,length(ran1),by=2)],ran1[seq(2,length(ran1),by=2)],pch=".",
     xlab="even random values",ylab="odd random values",
     main="Random values from ran1")
###

                                        # Fig. 3.4
###
### not in the book
plot(ran2[seq(1,length(ran2),by=2)],ran2[seq(2,length(ran2),by=2)],pch=".",
     xlab="even random values",ylab="odd random values",
     main="Random values from ran2")
###

rserial <- function(x,y) {
  n <- length(x)
  ck <- sum((x[1:(n-k)]-mean(x))*(x[(1+k):n]-mean(x)))
  c0 <- sum((x-mean(x))^2)
  ck/c0
}

round(c(rserial(ran0,2),rserial(ran1,2),rserial(ran2,2)),3)
round(c(rserial(ran0,3),rserial(ran1,3),rserial(ran2,3)),3)
round(c(rserial(ran0,4),rserial(ran1,4),rserial(ran2,4)),3)
round(c(rserial(ran0,5),rserial(ran1,5),rserial(ran2,5)),3)

                                        # Fig. 3.5
###
### not in the book
x <- c(0.25,0.84,0.39,0.59,0.76,0.40,0.26,0.41,0.47,
       0.75,0.40,0.34,0.72,0.61,0.31,0.90,0.90,0.82)
y <- c(0.47,0.83,0.38,0.80,0.67,0.68,0.57,0.63,0.38,
       0.12,0.21,0.28,0.36,0.87,0.28,0.24,0.88,0.14)
x0 <- 0.60
y0 <- 0.65
plot(x,y,pch="+",main="Distance data")
points(x0,y0,pch=19,cex=1.4)
text(x0,y0+.05,"(x0,y0)",pos=4)
###

d0 <- sqrt((x-x0)^2+(y-y0)^2)
mean(d0)

xx <- matrix(runif(5000*18),18,5000)
yy <- matrix(runif(5000*18),18,5000)
d <- sqrt((xx-x0)^2+(yy-y0)^2)
dbar <- apply(d,2,mean)

                                        # Fig. 3.6
###
### not in the book
hist(dbar,ylab="",main="Histogram of simulated mean values (n = 18)")
###

z <- (mean(d0)-mean(dbar))/sqrt(var(dbar))
pvalue <- pnorm(z)
cbind(z,pvalue)

n <- 2000
rolls <- matrix(ceiling(6*runif(2*n)),2,n)
tosses <- apply(rolls,2,sum)
o <- table(tosses)/n
e <- c(1,2,3,4,5,6,5,4,3,2,1)/36
round(cbind(0,e),4)

                                        # Fig. 3.7
###
### not in the book
p <- c(0.4,0.3,0.2,0.1)
draw.rect <- function(x,y) {
  rect(x[1],0,x[2],y)
  points(c(x[1],x[2]),c(y,y),pch=19,cex=1)
  text((x[1]+x[2])/2,y/2,paste("area=",y,sep=""))
}
plot(0:5,type="n",xlim=c(0,5),ylim=c(0,0.5),
     xlab="x-values",ylab="p=probabilities")
draw.rect(c(0.5,1.5),p[1])
draw.rect(c(1.5,2.5),p[2])
draw.rect(c(2.5,3.5),p[3])
draw.rect(c(3.5,4.5),p[4])
lines(c(1.5,4.5),c(p[1],p[1]),lty=3)
lines(c(4.5,4.5),c(p[1],0),lty=3)
text(3,p[2],"area=0.6")
title("Simple probability distribution")
###

n <- 10000
p <- c(4,3,2,1)/10
p
u <- ceiling(4*runif(n))
ran1 <- ifelse(runif(n)<p[u]/max(p),u,999)
sum(ifelse(ran1==999,0,1))/n
ran1 <- ran1[ran1!=999]
ran1 <- ran1[1:5000]
table(ran1)/5000
mean(ran1)
var(ran1)
ztest <- (mean(ran1)-2)*sqrt(n)
ztest
2*(1-pnorm(abs(ztest)))

x0 <- rep(1:4,4:1)
ran2 <- sample(x0,5000,replace=T)
table(ran2)/5000

n <- 5000
p <- c(4,3,2,1)/10
p
f <- c(0,cumsum(p))
f
### - R --
### change ran2 to numeric vector
ran2 <- as.numeric(cut(runif(n),f,labels=c(1,2,3,4)))
table(ran2)/n
mean(ran2)
var(ran2)
ztest <- (mean(ran2)-2)*sqrt(n)
ztest
2*(1-pnorm(abs(ztest)))
  
                                        # Fig. 3.8
###
### not in the book
plot(f[-1],type="h",ylim=c(0,1),xlab="x",ylab="p=probabilities")
points(1:4,f[-1],pch="+")
title("Cumulative probabilities for x = 1, 2, 3 and 4")
###

n <- 10
p <- 0.4
x <- 0:n
round(dbinom(x,n,p),3)

choose <- function(n,x) { gamma(n+1)/(gamma(x+1)*gamma(n-x+1)) }
round(choose(n,x)*p^x*(1-p)^(n-x),3)

round(pbinom(x,n,p),3)

round(cumsum(dbinom(x,n,p)),3)

px <- dbinom(x,n,p)
ex <- sum(x*px)
v <- sum((x-ex)^2*px)
cat("expectation =",round(ex,3)," variance =",round(v,3),"\n")

rbinom(20,n,p)

k <- 20
rtemp <- matrix(ifelse(runif(n*k)<p,1,0),n,k)
apply(rtemp,2,sum)

prop.test(794,1025,conf.level=0.95)$conf.int

x <- c(794,1077)
n <- c(1025,1478)
prop.test(x,n,conf.level=0.99)

prop.test(x,n,conf.level=0.99)$conf.int

prop.test(x,n)$p.value

N <- 20
m <- 10
n <- 6
x <- max(0,n+m-N):min(m,n)
round(dhyper(x,m,N-m,n),3)
round(phyper(x,m,N-m,n),3)
round(cumsum(dhyper(x,m,N-m,n)),3)

px <- dhyper(x,m,N-m,n)
ex <- sum(x*px)
v <- sum((x-ex)^2*px)
cat("expectation =",round(ex,3)," variance =",round(v,3),"\n")

hprob <- function(N,m,n) {
  xtemp <- c(rep(0,N-m),rep(1,m))
  id <- rank(runif(N))[1:n]
  sum(xtemp[id])
}

###
### not in the book
nsample <- 20
###

rhyper(nsample,m,N-m,n)

x <- 0:10
lam <- 2
round(dpois(x,lam),3)
round(ppois(x,lam),3)

x <- 0:30
px <- dpois(x,lam)
ex <- sum(x*px)
v <- sum((x-ex)^2*px)
cat("expectation =",round(ex,3)," variance =",round(v,3),"\n")

f <- c(57,203,383,525,532,408,273,139,45,43)
n <- length(f)
x <- rep(0:(n-1),f)
table(x)
lam <- mean(x)
lam
p <- dpois(0:(n-2),lam)
p[n] <- 1-sum(p)
ex <- round(sum(f)*p,1)
cbind(f,ex)

n <- 10000
lam <- 2
u <- ceiling(10*runif(n))
p <- dpois(0:10,lam)
round(p,3)
x <- ifelse(runif(n)<p[u]/max(p),u-1,999)
x <- x[x!=999]
x <- x[1:2000]
mean(x)
var(x)

n <- 2000
lam <- 2
pp <- c(0,ppois(0:10,lam),1)
round(pp,3)
x <- cut(runif(n),pp)
x <- x-1
mean(x)
var(x)

n <- 2000
lam <- 2
x <- rpois(n,lam)
mean(x)
var(x)

x <- 0:10
p <- 0.5
round(dgeom(x,p),3)
round(pgeom(x,p),3)

x <- 0:30
px <- dgeom(x,p)
ex <- sum(x*px)
v <- sum((x-ex)^2*px)
cat("expectation =",round(ex,3)," variance =",round(v,3),"\n")

n <- 20
p <- 0.5
x <- trunc(log(runif(n))/log(1-p))
x

rgeom(n,p)

                                        # Fig. 3.9
###
### not in the book
library(triangle)
par(mfrow=c(2,2))
plot(seq(0,1,length=101),dtriangle(seq(0,1,by=.01),0,1,1),type="l",
     xlab="x",ylab="f(x)",main="Triangular density function")
plot(seq(0,1,length=101),cumsum(dtriangle(seq(0,1,by=.01),0,1,1))/100,
     type="l",xlab="x",ylab="F(x)",main="Triangular cumulative function")
xx <- seq(-4,4,by=.05)
plot(xx,dnorm(xx),type="l",xlab="x",ylab="f(x)",
     main="Normal density function")
plot(xx,cumsum(dnorm(xx)),type="l",xlab="x",ylab="F(x)",
     main="Normal cumulative function")
###

moments <- function(x0,xn,k,f) {
  b <- c(7,rep(c(32,12,32,14),k))
  b[length(b)] <- 7
  x <- seq(x0,xn,length=length(b))
  h <- x[2]-x[1]
  area <- sum((2*h/45)*b*f(x))
  mean <- sum((2*h/45)*b*x*f(x))
  variance <- sum((2*h/45)*b*(x-mean)^2*f(x))
  round(cbind(area,mean,variance),3)
}

f <- function(x) { exp(-.5*(x-1)^2/2)/sqrt(2*pi*2) }
moments(-10,10,10,f)
f <- function(x) { exp(-.5*(x-3)^2/0.5)/sqrt(2*pi*0.5) }
moments(-10,10,30,f)
f <- function(x) { 2*x }
moments(0,1,10,f)
f <- function(x) { 1 }
moments(0,1,10,f)
f <- function(x) { 1/5 }
moments(0,5,5,f)
f <- function(x) { 0.5*exp(-0.5*x) }
moments(0,50,50,f)

                                        # Fig. 3.10
###
### not in the book
u <- c(0.89,0.67,0.78,0.51,0.34,0.40,0.74,0.61,0.98,0.84)
f <- function(x) { -log(1-x)/0.5 }
x <- f(u)
xx <- f(seq(0,1,by=.01))
plot(xx,seq(0,1,by=.01),type="l",xlab="x",ylab="u=F(x)",xlim=c(0,10))
## points(x,u,pch="+")
abline(h=0,col="lightgray")
abline(v=0,col="lightgray")
for (i in 1:10) {
  arrows(0,u[i],x[i],u[i],length=.05)
  arrows(x[i],u[i],x[i],0,length=.05)
}
title("Random exponential values")
###

x <- rnorm(100,4,2)
x <- 2*rnorm(100)+4

n <- 1000
z0 <- rnorm(n)
round(c(mean(z0),var(z0)),3)

z1 <- qnorm(runif(n))
round(c(mean(z1),var(z1)),3)

ztemp <- sqrt((-pi/2)*log(1-(2*runif(n)-1)^2))
z2 <- ifelse(runif(n)<0.5,-ztemp,ztemp)
round(c(mean(z2),var(z2)),3)

t <- sqrt(log(1/runif(n)^2))
temp1 <- 2.515517+0.802853*t+0.010328*t^2
temp2 <- 1+1.432788*t+0.189269*t^2+0.001308*t^3
ztemp <- t-temp1/temp2
z3 <- ifelse(runif(n)<0.5,-ztemp,ztemp)
round(c(mean(z3),var(z3)),3)

u1 <- runif(n)
u2 <- runif(n)
z4 <- sqrt(-2*log(u1))*cos(2*pi*u2)
z5 <- sqrt(-2*log(u1))*sin(2*pi*u2)
round(c(mean(z4),var(z4)),3)
round(c(mean(z5),var(z6)),3)

temp <- matrix(runif(12*n),12,n)
z6 <- apply(temp,2,sum)-6
round(c(mean(z6),var(z6)),3)

u1 <- runif(5*n,-3,3)
u2 <- runif(5*n)
x <- ifelse(u2<exp(-0.5*u1^2),u1,999)
z7 <- x[x!=999][1:n]
round(c(mean(z7),var(z7)),3)

ranchi2 <- function(n,df) {
  mtemp <- matrix(rnorm(n*df)^2,n,df)
  apply(mtemp,1,sum)
}
x <- ranchi2(10000,4)
mean(x)
var(x)

n <- 1000
m <- log(1.5)
s <- log(2)
x <- exp(m+s*rnorm(n))
mean(log(x))
sqrt(var(log(x)))
median(x)
mean(x)

n <- 6
u <- runif(1000)
y1 <- qnorm(1-u^(1/n))
u <- runif(1000)
yn <- qnorm(u^(1/n))
round(cbind(n,var(y1),var(yn)),3)

n <- 4
nvalues <- 10
m <- log(2.0)
s <- log(1.5)
rvalues <- qlnorm(1-runif(nvalues)^(1/n),m,s)
round(rvalues,3)

                                        # Fig. 3.11
###
### not in the book
x <- c(-2.5,-1.28,-0.842,-0.524,-0.253,0,0.253,0.524,0.842,1.28,2.5)
xx <- seq(-4,4,by=.05)
plot(xx,dnorm(xx),type="l",xlab="z",ylab="",xlim=c(-2.5,2.5))
abline(h=0,col="lightgray")
points(x,c(0,dnorm(x[-c(1,11)]),0),pch=19)
points(x[seq(3,9,1)][-4],dnorm(x[c(2,3,4,8,9,10)]),pch=19)
segments(x[c(2,3,4,5,7,8,9)],dnorm(x[c(2,3,4,5,8,9,10)]),
         x[c(3,4,5,7,8,9,10)],dnorm(x[c(2,3,4,5,8,9,10)]),lty=3)
segments(x[-c(1,11)],rep(0,9),x[-c(1,11)],dnorm(x[-c(1,11)]),lty=3)
segments(x[1],0,x[2],dnorm(x[2]),lty=3)
segments(x[11],0,x[10],dnorm(x[10]),lty=3)
title("Normal distribution")
###

###
### not in the book
v <- matrix(c(9,3,3,4),2,2)
###

v
a <- chol(v)
t(a)%*%a

###
### not in the book
k <- 4
v <- matrix(c(10,1,2,1,1,4,2,1,2,2,3,1,1,1,1,8),k)
###

mu <- c(1,2,3,4)
z <- rnorm(k)
t <- chol(v)
x <- t(chol(v))%*%z+mu
round(cbind(mu,x),3)
